import pygame, random, math
import Player, TileSystem, People, Globals

screen = None
SCREEN_WIDTH = 256
SCREEN_HEIGHT = 600
TOTAL_LEVELS = 6
NETSPEED = 100
rimage = None
rshadow = None
tiles = None
pimage = None
pshadow = None
people_sounds = []
img_caught = None
img_escape = None
img_win = None
img_victory = None
img_gameover = None
img_credits = None
img_mm = None
img_menu = []
img_life = None
img_net = []


def load_image(filename, colorkey = False):
    image = pygame.image.load(filename)
    if colorkey == True:
        image.set_colorkey((255,0,255), pygame.RLEACCEL)
        return image.convert()
    elif colorkey == "alpha":
        return image.convert_alpha()
    else:
        return image.convert()


def init_pygame():
    global screen
    pygame.init()
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), 0, 32)
    pygame.display.set_caption('run rabbit, RUN')
    pygame.mixer.init(11025)

def load_data():
    global rimage,tiles, rshadow, people_sounds, pimage, pshadow
    global img_menu, img_life, img_win, img_caught,img_escape, img_mm
    global img_net,img_gameover,img_victory,img_credits
    rimage = load_image("data/Rabbit/run.png",True)
    tiles = load_image("data/tiles.png")
    rshadow = load_image("data/Rabbit/shadow.png",True)
    pimage = load_image("data/people/walk.png","alpha")
    pshadow = load_image("data/people/shadow.png",True)
    img_caught = load_image("data/caught2.png",False)
    img_escape = load_image("data/escape.png",False)
    img_win = load_image("data/win.png",False)
    img_mm = load_image("data/mm.png",False)
    img_credits = load_image("data/credits.png",False)
    img_gameover = load_image("data/gameover.png",False)
    img_victory = load_image("data/victory.png",False)
    img_life = load_image("data/Rabbit/life.png","alpha")
    #caught, near, middle, far, escape
    people_sounds = [[pygame.mixer.Sound("data/sfx/people/caught/all.wav"),
                      pygame.mixer.Sound("data/sfx/people/caught/1.wav"),
                      pygame.mixer.Sound("data/sfx/people/caught/2.wav"),
                      pygame.mixer.Sound("data/sfx/people/caught/3.wav"),
                      pygame.mixer.Sound("data/sfx/people/caught/4.wav"),
                      pygame.mixer.Sound("data/sfx/people/caught/5.wav")],

                     [pygame.mixer.Sound("data/sfx/people/near/1.wav"),
                      pygame.mixer.Sound("data/sfx/people/near/2.wav"),
                      pygame.mixer.Sound("data/sfx/people/near/3.wav"),
                      pygame.mixer.Sound("data/sfx/people/near/4.wav"),
                      pygame.mixer.Sound("data/sfx/people/near/5.wav"),
                      pygame.mixer.Sound("data/sfx/people/near/6.wav")],

                     [pygame.mixer.Sound("data/sfx/people/mid/1.wav"),
                      pygame.mixer.Sound("data/sfx/people/mid/2.wav"),
                      pygame.mixer.Sound("data/sfx/people/mid/3.wav"),
                      pygame.mixer.Sound("data/sfx/people/mid/4.wav"),
                      pygame.mixer.Sound("data/sfx/people/mid/5.wav"),
                      pygame.mixer.Sound("data/sfx/people/mid/6.wav")],
                     
                     [pygame.mixer.Sound("data/sfx/people/far/1.wav"),
                      pygame.mixer.Sound("data/sfx/people/far/2.wav"),
                      pygame.mixer.Sound("data/sfx/people/far/3.wav"),
                      pygame.mixer.Sound("data/sfx/people/far/4.wav"),
                      pygame.mixer.Sound("data/sfx/people/far/5.wav"),
                      pygame.mixer.Sound("data/sfx/people/far/6.wav")],

                     [pygame.mixer.Sound("data/sfx/people/escape/all.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/1.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/2.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/3.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/4.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/5.wav"),
                      pygame.mixer.Sound("data/sfx/people/escape/6.wav")],
                     
                     [pygame.mixer.Sound("data/sfx/people/normal/r.ogg")]]
    img_menu = [
        load_image("data/continue.png","alpha"),
        0,
        load_image("data/quit.png","alpha"),
        load_image("data/retry.png","alpha"),
        load_image("data/startover.png","alpha"),
        load_image("data/lives.png","alpha"),
        load_image("data/play.png","alpha"),
        load_image("data/quitgame.png","alpha")]
    img_net = [
        load_image("data/net.png","alpha"),
        load_image("data/snet.png","alpha")]

class Net(object):
    def __init__(self,startx, starty, targetx,targety):
        self.x = startx
        self.y = starty
        self.z = 30
        self.tx = targetx
        self.ty = targety
        self.dx = float((targetx-startx)/(NETSPEED*2))
        self.dy = float((starty-targety)/(NETSPEED*2))
        self.state = 0
        self.angle = 0
        self.rot = random.randrange(100, 1500)
            
    def update(self, dt):
        if self.state == 0:
            self.x += NETSPEED*dt*self.dx
            self.y -= NETSPEED*dt*self.dy

            if abs(self.x-self.tx) < 16 and self.y-self.ty < 0:
                self.state = 1
            self.angle += self.rot*dt
                
        elif self.state == 1:
            self.z -= 100*dt
            if self.z < 0:
                self.z = 0
                self.state = 2
            self.angle += self.rot*dt
        elif self.state == 2:
            self.y += Globals.SCROLLSPEED*dt

        if self.angle > 360:
            self.angle = 0
        if self.angle < 0:
            self.angle = 360
            

        

        if self.y+16 < 0 or self.y-16 > 600:
            return -1
        if self.x+16 < 0 or self.x-16 > 256:
            return -1

        return 0

    def draw(self):
        im1 = pygame.transform.rotate(img_net[1], self.angle)
        im2 = pygame.transform.rotate(img_net[0], self.angle)
        w = im1.get_rect().w/2
        screen.blit(im1,(self.x-w,self.y-w))
        screen.blit(im2,(self.x-w,(self.y-w)-self.z))
        
        
        


def play(lives, level):
    state = 0
    over = False
    t = TileSystem.TileSystem("data/level"+str(level)+".map",32,32,tiles)
    t.load()
    h = float((t.mh*t.tileh)-600)

    last_time = pygame.time.get_ticks()

    p = Player.Player(128, 400,rimage,rshadow)

    fade = 0
    img_caught.set_alpha(0)

    pm = People.Manager(people_sounds,pimage, pshadow)

    nets = []
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_UP:
                    p.move(0,-1)
                if event.key == pygame.K_DOWN:
                    p.move(0,1)
                if event.key == pygame.K_LEFT:
                    p.move(-1,0)
                if event.key == pygame.K_RIGHT:
                    p.move(1,0)
                if event.key in Globals.JUMP_BUTTONS:
                    p.jump()
                #CHEAT
                if event.key == pygame.K_w:
                    h = 0

            if event.type == pygame.KEYUP:
                if event.key == pygame.K_UP:
                    p.move(0,1)
                if event.key == pygame.K_DOWN:
                    p.move(0,-1)
                if event.key == pygame.K_LEFT:
                    p.move(1,0)
                if event.key == pygame.K_RIGHT:
                    p.move(-1,0)

        delta = float(pygame.time.get_ticks() - last_time)
        delta /= 1000.0

        for i in range(len(nets)):
            if i >= len(nets):
                    break
            r = nets[i].update(delta)
            if r == -1:
                del nets[i]
            else:
                if nets[i].state == 1 and nets[i].z < 15:
                    if math.sqrt(((nets[i].x-p.x)*(nets[i].x-p.x))+((nets[i].y-p.y)*(nets[i].y-p.y))) < 16:
                        p.state = 1
                        p.bk = 4
                        del nets[i]
                                 
        if level == 2:
            if len(nets) == 0 and p.y < 200:
                nets.append(Net(200,530, p.x,p.y))
        if level == 3:
            if len(nets) == 0 and p.y < 300:
                nets.append(Net(200,530, p.x,p.y))
        if level == 4:
            if len(nets) == 0 and p.y < 400:
                nets.append(Net(200,530, p.x,p.y))
        if level == 5:
            if len(nets) == 0 and p.y < 600:
                nets.append(Net(200,530, p.x,p.y))
        if level == 6:
            if len(nets) == 0 and p.y < 600:
                nets.append(Net(200,530, p.x,p.y))
        
        #if len(nets) == 0 and p.y < 300:
            #nets.append(Net(200,530, p.x,p.y))
            #for nx in range(level):
            #    nets.append(Net(200,530, p.x-((nx-level/2)*32),p.y))
            
                #nets.append(Net(200,530, p.x,p.y))
            #if level == 4
                    
                                        
            
        

        p.update(delta,t.getColl, h)

        

        pm.update(p.x,p.y,h,delta, state)
        

        h -= (delta * Globals.SCROLLSPEED)
        if h < 0:
            h = 0


        if h == 0:
            state = 2
            p.state = 2
        elif p.y > 520:
            state = 1
            p.state = 3

        if state != 0:
            fade += 10
            if fade > 255:
                fade = 255
                img_caught.set_alpha(255)
                img_escape.set_alpha(255)
                img_victory.set_alpha(255)
                img_gameover.set_alpha(255)
                if state == 1:
                    if lives-1 == 0:
                        screen.blit(img_gameover,(0,0))
                    else:
                        screen.blit(img_caught,(0,0))
                elif state == 2:
                    if level == TOTAL_LEVELS:
                        screen.blit(img_victory,(0,0))
                    else:
                        screen.blit(img_escape,(0,0))
                return state
            img_caught.set_alpha(fade)
            img_escape.set_alpha(fade)
            img_win.set_alpha(fade)
            img_gameover.set_alpha(fade)
            img_victory.set_alpha(fade)
        

        last_time = pygame.time.get_ticks()
        #drawing
        t.draw(int(h))
        for net in nets:
            if net.state == 2:
                net.draw()
        p.draw()
        pm.draw()
        for net in nets:
            if net.state < 2:
                net.draw()
        for i in range(lives):
            screen.blit(img_life,((i*32),0))
        if state == 1:
            if lives-1 == 0:
                screen.blit(img_gameover,(0,0))
            else:
                screen.blit(img_caught,(0,0))
        elif state == 2:
            if level == TOTAL_LEVELS:
                screen.blit(img_victory,(0,0))
            else:
                screen.blit(img_escape,(0,0))
        pygame.display.flip()

def caught_menu(lives):
    over = False
    #0 - Main Menu
    #1 - Continue
    #2 - Retry
    #3 - restart
    if lives == 0:
        screen.blit(img_menu[4],(0,400))
        screen.blit(img_menu[2],(0,500))

        pygame.mixer.stop()
        c = people_sounds[0][0].play()
        c.set_volume(0.4)
        
    if lives > 0:
        pygame.mixer.stop()
        screen.blit(img_menu[5],(0,300))
        screen.blit(img_menu[3],(0,400))
        screen.blit(img_menu[2],(0,500))
        for i in range(lives):
            screen.blit(img_life,((i*32),350))
        c = people_sounds[0][random.randrange(1,4)].play()
        c.set_volume(0.4)
    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return 0
                if event.key in Globals.JUMP_BUTTONS:
                    return 2
        if pygame.mouse.get_pressed()[0]:
                if lives == 0:
                    if pygame.mouse.get_pos()[1] > 500:
                        return 0
                    elif pygame.mouse.get_pos()[1] > 400:
                        return 3
                if lives > 0:
                    if pygame.mouse.get_pos()[1] > 500:
                        return 0
                    elif pygame.mouse.get_pos()[1] > 400:
                        return 2
                
            
        pygame.display.flip()
        
def escape_menu(lives):
    over = False
    #      type, ret
    ret = -1
    screen.blit(img_menu[5],(0,300))
    screen.blit(img_menu[0],(0,400))
    screen.blit(img_menu[2],(0,500))
    c = people_sounds[4][random.randrange(1,4)].play()
    c.set_volume(0.4)
    for i in range(lives):
        screen.blit(img_life,((i*32),350))
    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return 0
                if event.key in Globals.JUMP_BUTTONS:
                    return 1
        if pygame.mouse.get_pressed()[0]:
                if pygame.mouse.get_pos()[1] > 500:
                    return 0
                elif pygame.mouse.get_pos()[1] > 400:
                    return 1
            
        pygame.display.flip()
        

def win_menu(lives):
    over = False
    #0 - Main Menu
    #1 - Continue
    #2 - Retry
    #3 - restart

    screen.blit(img_victory, (0,0))
    
    ret = -1
    #screen.blit(img_win,(0,0))
    #screen.blit(img_menu[5],(0,300))
    #screen.blit(img_menu[2],(0,500))
    #for i in range(lives):
        #screen.blit(img_life,((i*32),350))

    c = people_sounds[4][0].play()
    c.set_volume(0.2)

    m = False
    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return 0
                if event.key in Globals.JUMP_BUTTONS:
                    screen.blit(img_win,(0,0))
                    screen.blit(img_menu[5],(0,300))
                    screen.blit(img_menu[2],(0,500))
                    for i in range(lives):
                        screen.blit(img_life,((i*32),350))
                    m = True
        if pygame.mouse.get_pressed()[0] and m:
                if pygame.mouse.get_pos()[1] > 500:
                    return 0
            
        pygame.display.flip()

def help_menu():
    over = False
    #0 - Main Menu
    #1 - Continue
    #2 - Retry
    #3 - restart
    ret = -1

    bgs = [
        [load_image("data/help/1_movement.png","alpha"),
         load_image("data/help/1_1_keys.png","alpha"),
         load_image("data/help/1_2_forward.png","alpha"),
         load_image("data/help/1_3_bushes.png","alpha")],

        [load_image("data/help/2_jumping.png","alpha"),
         load_image("data/help/2_1_jumping.png","alpha"),
         load_image("data/help/2_2_low.png","alpha")],
    
        [load_image("data/help/3_jumping.png","alpha"),
         load_image("data/help/3_1_high.png","alpha")],
        
        [load_image("data/help/4_mud.png","alpha"),
         load_image("data/help/4_1_mud.png","alpha")],
        
        [load_image("data/help/5_nets.png","alpha")],
        
        [load_image("data/help/6_nets.png","alpha")],
        
        [load_image("data/help/7_endgame.png","alpha"),
         load_image("data/help/7_1_lose.png","alpha"),
         load_image("data/help/7_2_win.png","alpha")],
        
        [load_image("data/help/tips.png","alpha")]]

    c = 0
    sc = 1
    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return 1
                else:
                    sc += 1
                    if sc >= len(bgs[c]):
                        sc = 1
                        c += 1
                        if c >= len(bgs):
                            return 1
                        if len(bgs[c]) == 1:
                            sc = 0
                    
        screen.blit(bgs[c][0],(0,0))
        screen.blit(bgs[c][sc],(0,0))
        
        pygame.display.flip()


def credit_menu():
    over = False
    #0 - Main Menu
    #1 - Continue
    #2 - Retry
    #3 - restart
    ret = -1

    screen.blit(img_credits, (0,0))
    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return -1
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return 1
                else:
                    return 1
        
        pygame.display.flip()

def menu():
    over = False
    ret = -1
    screen.blit(img_mm,(0,0))
    screen.blit(img_menu[6],(0,400))
    screen.blit(img_menu[7],(0,500))
    pygame.mixer.stop()    
    while not over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return 0
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_h:
                    ret = help_menu()
                    if ret == -1:
                        return ret
                    screen.blit(img_mm,(0,0))
                    screen.blit(img_menu[6],(0,400))
                    screen.blit(img_menu[7],(0,500))
                if event.key in Globals.JUMP_BUTTONS:
                    return 1
                if event.key == pygame.K_q:
                    return 0
                if event.key == pygame.K_c:
                    ret = credit_menu()
                    if ret == -1:
                        return ret
                    screen.blit(img_mm,(0,0))
                    screen.blit(img_menu[6],(0,400))
                    screen.blit(img_menu[7],(0,500))
            if event.type == pygame.MOUSEBUTTONDOWN:
                if pygame.mouse.get_pos()[1] > 500:
                    return 0
                elif pygame.mouse.get_pos()[1] > 400:
                    return 1
            
        pygame.display.flip()
    

def main(volume = 1):
    init_pygame()
    load_data()

    lives = 8
    gr = 2
    level = 1

    #pygame.mixer.set_volume(volume)

    #game
    while(gr != 0):

        if gr == 2:
            ret = menu()
            if ret == -1 or ret == 0:
                return
            level = 1
            lives = 8
        
        ret = play(lives, level)

        if ret == -1 or ret == 0:
            return
        if ret == 1:
            lives -= 1
            r2 = caught_menu(lives)
            if r2 == -1:
                gr = 0
            elif r2 == 0:
                gr = 2
            elif r2 == 3:
                lives = 8
                level = 1
            elif r2 == 2:
                gr = 1
        if ret == 2:
            level += 1
            if level > TOTAL_LEVELS:
                #win
                r2 = win_menu(lives)
                if r2 == -1:
                    gr = 0
                if r2 == 0:
                    gr = 2
            else:
                r2 = escape_menu(lives)
                if r2 == -1:
                    gr = 0
                if r2 == 0:
                    gr = 2
                elif r2 == 1:
                    gr = 1



    
